# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

sys.path.insert(0, os.path.abspath("."))
sys.path.insert(0, os.path.abspath("../"))

import zoro_ds_utils

# -- Project information -----------------------------------------------------

project = zoro_ds_utils.__name__
copyright = zoro_ds_utils.__copyright__
author = zoro_ds_utils.__author__

# The full version, including alpha/beta/rc tags
release = zoro_ds_utils.__version__


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.doctest",
    "sphinx.ext.napoleon",
    "sphinxcontrib.apidoc",
    "sphinxcontrib.confluencebuilder",
]

# Defaults to only .rst
source_suffix = source_suffix = {
    ".rst": "restructuredtext",
    ".md": "markdown",
}

# The root toctree document.
root_doc = "index"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# Specify the language the docs are written in.
language = "en"


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "alabaster"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]


# -- Options for autodoc extension ----------------------------------------------

# Include Python objects as they appear in source files
# http://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html#confval-autodoc_member_order
# Default: alphabetically ('alphabetical' or 'groupwise' or 'bysource')
autodoc_member_order = "groupwise"

# Default flags used by autodoc directives
# autodoc_default_flags = ['members', 'undoc-members', 'private-members',
#                          'special-members', 'show-inheritance']

autodoc_inherit_docstrings = (
    True  # This is already the default behavior; here for explicitness
)


# -- Options for autosummary extension ----------------------------------------------

# Generate autodoc stubs with summaries from code
autosummary_generate = True


# -- Options for napoleon extension ----------------------------------------------

napoleon_google_docstring = True
napoleon_numpy_docstring = False
napoleon_include_init_with_doc = False
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = True


# -- Options for apidoc extension ----------------------------------------------

# Auto runs the equivalent of sphinx-apidoc -o ./docs/_source ./zoro_ds_utils -f on build
# See https://github.com/sphinx-contrib/apidoc#configuration
apidoc_module_dir = "../zoro_ds_utils"
apidoc_output_dir = "_source"
apidoc_excluded_paths = []
apidoc_separate_modules = True
# apidoc_toc_file = None
# apidoc_module_first = False
# apidoc_extra_args = []


# -- Options for Confluence Builder -------------------------------------------------
# See https://sphinxcontrib-confluencebuilder.readthedocs.io/en/stable/configuration/

# Essential Configuration
confluence_publish = True
confluence_space_key = "AKB"  # Analytics Knowledge Base
confluence_server_url = "https://zorotools.atlassian.net/wiki/"
confluence_server_user = os.getenv("CONFLUENCE_SERVER_USER")
# This should be an API token, created at https://id.atlassian.com/manage-profile/security/api-tokens
confluence_server_pass = os.getenv("CONFLUENCE_SERVER_PASS")
# Token version doesn't appear to work as documented
# confluence_publish_token = os.getenv("CONFLUENCE_PUBLISH_TOKEN")

# Generic Configuration
confluence_page_hierarchy = True

# Publishing Configuration
confluence_parent_page = "zuds library"
confluence_parent_page_id_check = 3145760943
# Ensures unique confluence page names, otherwise unrelated pages might get overwritten
confluence_publish_postfix = "(zuds)"
# confluence_sourcelink



"""Generic Python utilities."""
from typing import Any, Dict, List


def safe_nested_get(d: Dict, keys: List, default=None) -> Any:
    """Get a nested key's value from a dictionary.

    If the key doesn't exist, return `default` instead of raising a KeyError or TypeError.

    Args:
        d: A dictionary to search for 'keys'.
        keys: A list representing a nested dictionary key structure.
              E.g. safe_nested_get(d, keys=["a", "b", "c"] is a safe version of d["a"]["b"]["c"].
        default: The value to return if the nested `keys` structure doesn't exist in `d`.

    :Author: Zax Rosenberg
    """
    for key in keys:
        try:
            d = d[key]
        except (KeyError, TypeError):
            # KeyError: '<key>'
            # TypeError: '<type>' object is not subscriptable
            return default
    return d


def safe_nested_set(d: Dict, keys: List, value: Any) -> None:
    """Set a dictionary's `value` for a set of nested `keys`, inplace.

    If intermediate keys don't exist, they'll be created.
    In cases where `value` is a dictionary, and `keys` already exists,
    the previous value will be overwritten.
    To merge values instead, use `safe_nested_update`.

    Args:
        d: A dictionary to search for 'keys'.
        keys: A list representing a nested dictionary key structure.
              E.g. safe_nested_set(d, keys=["a", "b", "c"], "foo") is a safe version of d["a"]["b"]["c"] = "foo".
        value: The value to set.

    :Author: Zax Rosenberg
    """
    for key in keys[:-1]:
        d = d.setdefault(key, {})
    d[keys[-1]] = value


def safe_nested_update(d: Dict, updates: Dict) -> None:
    """Update a dictionary's contents with values from another nested dictionary, inplace.

    This method avoids overwriting the lowest level key when there are collisions, instead merging them.
    To overwrite on collisions, use `safe_nested_set`.

    Args:
        d: A dictionary to update with `updates`.
        updates: A dictionary from which to take values to add to `d`.

    :Author: Zax Rosenberg
    """
    for k, v in updates.items():
        if isinstance(v, dict):
            safe_nested_update(d.setdefault(k, {}), v)
        else:
            d[k] = v



# -*- coding: utf-8 -*-

"""Top-level package for zoro_ds_utils."""
import datetime
import pkgutil


__name__ = "zoro_ds_utils"
__version__ = pkgutil.get_data(__name__, "VERSION").decode()  # type: ignore[union-attr]
__author__ = "Zoro Data Science"
__email__ = ""
__copyright__ = f"Copyright (C) {datetime.datetime.now().year} Zoro"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""
from setuptools import setup

import zoro_ds_utils


# Avoid setuptools as an entrypoint unless it's the only way to do it.
# Prefer setup.cfg to setup.py unless not possible.
# Keep setup.py for easy "editable" installs, even if otherwise empty.
setup(
    name=zoro_ds_utils.__name__,
    author=zoro_ds_utils.__author__,
    author_email=zoro_ds_utils.__email__,
)



from typing import Any

import pytest

from zoro_ds_utils import general


def is_exception_type(value: Any) -> bool:
    """Returns True if `value` is a subclass of `Exception`, otherwise False."""
    try:
        # issubclass() throws a TypeError if `value` is not a class.
        if issubclass(value, Exception):
            return True
    except TypeError:
        pass

    return False


@pytest.mark.parametrize(
    "d, keys, default, expected",
    [
        ({}, ["foo"], None, None),  # KeyError
        ({"foo": 5}, ["foo"], None, 5),  # Success!
        ({"foo": 5}, ["foo", "bar"], None, None),  # TypeError
        ({"foo": 5}, ["foo"], -1, 5),  # Success  # Override default
        ({"foo": 5}, ["bar"], -1, -1),  # KeyError  # Override default
    ],
)
def test_safe_nested_get(d, keys, default, expected):
    assert general.safe_nested_get(d, keys, default) == expected


@pytest.mark.parametrize(
    "d, keys, value, expected",
    [
        ({}, ["foo"], "foovalue", {"foo": "foovalue"}),
        ({}, ["foo", "bar"], "barvalue", {"foo": {"bar": "barvalue"}}),
        (
            {"foo": "foovalue"},
            ["foo", "bar"],  # Can't replace existing str value with dict
            "barvalue",
            TypeError,
        ),
        (
            {"foo": {"value1": 3}},
            ["foo"],
            {"value2": 6},
            {"foo": {"value2": 6}},  # value1 was lost
        ),
    ],
)
def test_safe_nested_set(d, keys, value, expected):
    if not is_exception_type(expected):
        # General case: No exception thrown, check the function return value.
        general.safe_nested_set(d, keys, value)
        assert d == expected
    else:
        # Check for expected exception, e.g. TypeError
        with pytest.raises(expected):
            general.safe_nested_set(d, keys, value)


@pytest.mark.parametrize(
    "d, updates, expected",
    [
        ({}, {"foo": "foovalue"}, {"foo": "foovalue"}),
        (
            {"foo": {"value1": 3}},
            {"foo": {"value2": 6}},
            {"foo": {"value1": 3, "value2": 6}},  # Old key/values preserved
        ),
        (
            {"foo": {"value1": 3}},
            {"foo": {"value1": 4, "value2": 6}},
            {"foo": {"value1": 4, "value2": 6}},  # Update old, insert new
        ),
        (
            {"_zoro_mlops": {"first_attempt_start_date": "2020-05-10"}},
            {
                "MONITOR": {
                    "CPU": "1",
                    "ENV_VARS": {
                        "GOOGLE_PROJECT_BQ": "gcp-dw",
                        "RUN_ID": "2020.05.10.07.57.01-rc",
                        "STRATEGY_NAME": "similar_content",
                    },
                    "IMAGE": "gcr.io/zorodataplatform/ds-cc-product-recs:dev",
                    "MEMORY": "1G",
                    "RELEASE_TYPE": "dev",
                    "SCRIPT": "performance_monitoring.py",
                    "TIMEOUT": 1800,
                },
                "NOTIFICATIONS": {
                    "FAILURE": {"SLACK_IDS": ["G01J6646M16"]},
                    "SUCCESS": {"SLACK_IDS": ["G01J6646M16"]},
                },
                "RELEASE_TYPE": "dev",
            },
            {
                "_zoro_mlops": {"first_attempt_start_date": "2020-05-10"},
                "MONITOR": {
                    "CPU": "1",
                    "ENV_VARS": {
                        "GOOGLE_PROJECT_BQ": "gcp-dw",
                        "RUN_ID": "2020.05.10.07.57.01-rc",
                        "STRATEGY_NAME": "similar_content",
                    },
                    "IMAGE": "gcr.io/zorodataplatform/ds-cc-product-recs:dev",
                    "MEMORY": "1G",
                    "RELEASE_TYPE": "dev",
                    "SCRIPT": "performance_monitoring.py",
                    "TIMEOUT": 1800,
                },
                "NOTIFICATIONS": {
                    "FAILURE": {"SLACK_IDS": ["G01J6646M16"]},
                    "SUCCESS": {"SLACK_IDS": ["G01J6646M16"]},
                },
                "RELEASE_TYPE": "dev",
            },
        ),
    ],
)
def test_safe_nested_update(d, updates, expected):
    general.safe_nested_update(d, updates)
    assert d == expected
